"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var POPUP = _interopRequireWildcard(require("../constants/popup"));

var UI = _interopRequireWildcard(require("../constants/ui"));

var _message = require("../message");

var _builder = require("../message/builder");

var _DataManager = _interopRequireDefault(require("../data/DataManager"));

var _ConnectSettings = require("../data/ConnectSettings");

var _windowsUtils = require("../utils/windowsUtils");

var view = _interopRequireWildcard(require("./view"));

var _common = require("./view/common");

var _notification = require("./view/notification");

var _popup2 = _interopRequireDefault(require("../../styles/popup.less"));

var _LoaderInfo = require("../data/LoaderInfo");

var _installBridge = require("./view/installBridge");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// eslint-disable-next-line no-unused-vars
var olderThan = function olderThan(version1, version2) {
  if (version1[0] !== version2[0]) {
    return version1[0] < version2[0];
  }

  if (version1[1] !== version2[1]) {
    return version1[1] < version2[1];
  }

  return version1[2] < version2[2];
}; // handle messages from window.opener and iframe


var handleMessage = function handleMessage(event) {
  // console.log('------------------------ ');
  // console.log('event', event);
  var data = event.data;
  if (!data) return; // console.log('%c popup data type', 'color:#3a12dd', data.type);
  // console.log('data');
  // console.log(data);
  // if (data.payload) {
  //     console.log('popup payload', data.payload);
  // }
  // This is message from the window.opener

  if (data.type === POPUP.INIT) {
    init((0, _windowsUtils.escapeHtml)(data.payload)); // eslint-disable-line no-use-before-define

    return;
  } // This is message from the window.opener


  if (data.type === UI.IFRAME_FAILURE) {
    (0, _common.showView)('iframe-failure');
    return;
  } // ignore messages from origin other then MessagePort (iframe)


  var isMessagePort = event.target instanceof MessagePort || typeof BroadcastChannel !== 'undefined' && event.target instanceof BroadcastChannel;
  if (!isMessagePort) return; // catch first message from iframe

  if (data.type === POPUP.HANDSHAKE) {
    if (!data.payload.transport.version) {
      window.noBridge = true; // initInstallBridgeView()
      // return
    }

    handshake(data.payload); // eslint-disable-line no-use-before-define

    return;
  }

  var message = (0, _message.parseMessage)(data); // if (message) {
  //     console.log('popup message', message);
  // }

  switch (message.type) {
    case UI.LOADING:
      // case UI.REQUEST_UI_WINDOW :
      (0, _common.showView)('loader');
      break;

    case UI.SET_OPERATION:
      if (typeof message.payload === 'string') {
        (0, _common.setOperation)(message.payload);
      }

      break;

    case UI.TRANSPORT:
      (0, _common.showView)('transport');
      break;

    case UI.SELECT_DEVICE:
      view.selectDevice(message.payload);
      break;

    case UI.SELECT_ACCOUNT:
      view.selectAccount(message.payload);
      break;

    case UI.SELECT_FEE:
      view.selectFee(message.payload);
      break;

    case UI.UPDATE_CUSTOM_FEE:
      view.updateCustomFee(message.payload);
      break;

    case UI.INSUFFICIENT_FUNDS:
      (0, _common.showView)('insufficient-funds');
      break;

    case UI.REQUEST_BUTTON:
      view.requestButton(message.payload);
      break;

    case UI.BOOTLOADER:
      (0, _common.showView)('bootloader');
      break;

    case UI.NOT_IN_BOOTLOADER:
      (0, _common.showView)('not-in-bootloader');
      break;

    case UI.INITIALIZE:
      (0, _common.showView)('initialize');
      break;

    case UI.SEEDLESS:
      (0, _common.showView)('seedless');
      break;

    case UI.FIRMWARE_NOT_INSTALLED:
      (0, _common.showView)('firmware-install');
      break;
    // case UI.FIRMWARE_OLD:
    //     view.firmwareRequiredUpdate(message.payload);
    //     break;

    case UI.FIRMWARE_NOT_SUPPORTED:
      view.firmwareNotSupported(message.payload);
      break;

    case UI.FIRMWARE_NOT_COMPATIBLE:
      view.firmwareNotCompatible(message.payload);
      break;

    case UI.FIRMWARE_OUTDATED:
      // showFirmwareUpdateNotification(message.payload);
      break;

    case UI.DEVICE_NEEDS_BACKUP:
      (0, _notification.showBackupNotification)(message.payload);
      break;

    case UI.REQUEST_PERMISSION:
      view.initPermissionsView(message.payload);
      break;

    case UI.REQUEST_CONFIRMATION:
      view.initConfirmationView(message.payload);
      break;

    case UI.REQUEST_PIN:
      view.initPinView(message.payload);
      break;

    case UI.REQUEST_WORD:
      view.initWordView(message.payload);
      break;

    case UI.INVALID_PIN:
      (0, _common.showView)('invalid-pin');
      break;

    case UI.REQUEST_PASSPHRASE:
      view.initPassphraseView(message.payload); // window.postMessage(
      //     UiMessage(UI.RECEIVE_PASSPHRASE, {
      //         value: '',
      //         save: true,
      //     }),
      // );

      break;

    case UI.REQUEST_PASSPHRASE_ON_DEVICE:
      view.passphraseOnDeviceView(message.payload);
      break;

    case UI.INVALID_PASSPHRASE:
      view.initInvalidPassphraseView(message.payload);
      break;
    // no default
  }

  try {
    var features = data.payload.device.features;
    var major_version = features.major_version;
    var minor_version = features.minor_version;
    var patch_version = features.patch_version; // const major_version = 1, minor_version = 2, patch_version = 3
    // const major_version = 2, minor_version = 2, patch_version = 1
    // const major_version = 2, minor_version = 1, patch_version = 6

    var MIN_USABLE_VERSION = [2, 1, 6];

    if (olderThan([major_version, minor_version, patch_version], MIN_USABLE_VERSION)) {
      console.log('Firmware version under 2.1.6');
      (0, _installBridge.initInstallBridgeView)('firmware');
      return;
    }
  } catch (err) {
    console.error(err);
  } finally {
    if (window.noBridge) {
      (0, _installBridge.initInstallBridgeView)();
    }
  }
}; // handle POPUP.INIT message from window.opener


var init = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(payload) {
    var settings, useBroadcastChannel, broadcastID;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (payload) {
              _context.next = 2;
              break;
            }

            return _context.abrupt("return");

          case 2:
            settings = payload.settings; // npm version < 8.1.20 doesn't have it in POPUP.INIT message

            useBroadcastChannel = typeof payload.useBroadcastChannel === 'boolean' ? payload.useBroadcastChannel : true;
            _context.prev = 4;
            _context.next = 7;
            return _DataManager["default"].load((0, _ConnectSettings.parse)(settings), false);

          case 7:
            // initialize message channel
            broadcastID = useBroadcastChannel ? settings.env + "-" + settings.timestamp : undefined;
            (0, _common.initMessageChannel)(broadcastID, handleMessage); // reset loading hash

            window.location.hash = ''; // handshake with iframe

            view.initBrowserView();
            _context.next = 16;
            break;

          case 13:
            _context.prev = 13;
            _context.t0 = _context["catch"](4);
            (0, _common.postMessageToParent)((0, _builder.UiMessage)(POPUP.ERROR, {
              error: _context.t0.message
            }));

          case 16:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[4, 13]]);
  }));

  return function init(_x) {
    return _ref.apply(this, arguments);
  };
}(); // handle POPUP.HANDSHAKE message from iframe


var handshake = function handshake(payload) {
  if (!payload) return; // replace local settings with values from iframe (parent origin etc.)

  _DataManager["default"].settings = payload.settings;
  (0, _common.setOperation)(payload.method || '');

  if (payload.transport && payload.transport.outdated && payload.transport.version) {
    (0, _notification.showBridgeUpdateNotification)();
  }
};

var onLoad = function onLoad() {
  // view.initPinView()
  // request confirmation view
  // showBridgeUpdateNotification();
  // unsupported browser, this hash was set in parent app (PopupManager)
  // display message and do not continue
  // const payload = {
  //     // webusb: 'xxx',
  //     devices: [{
  //         type: 'unreadable',
  //         status: '',
  //         error: 'xxxxx'
  //     }, {
  //         type: 'unacquired',
  //         status: 'occupied',
  //         error: 'xxxxx'
  //     }, {
  //         type: 'acquired',
  //         status: 'occupied',
  //         error: 'xxxxx',
  //         label: 'LAZS'
  //     }, {
  //         type: 'unacquired',
  //         status: '',
  //         error: 'xxxxx'
  //     }, {
  //         type: 'acquired',
  //         status: '',
  //         error: 'xxxxx',
  //         label: 'YWJS'
  //     }, {
  //         type: 'unreadable',
  //         status: '',
  //         error: 'xxxxx'
  //     }, {
  //         type: 'unreadable',
  //         status: '',
  //         error: 'LIBUSB_ERROR'
  //     }, {
  //         type: 'unreadable',
  //         status: '',
  //         error: 'LIBUSB_ERROR',
  //         features: {
  //             major_version: 2
  //         }
  //     }]
  // }
  // view.selectDevice(payload);
  // return
  if (window.location.hash === '#unsupported') {
    view.initBrowserView(false);
    return;
  }

  (0, _common.postMessageToParent)((0, _builder.UiMessage)(POPUP.LOADED));
  (0, _LoaderInfo.showLoaderInfo)();
};

window.addEventListener('load', onLoad, false);
window.addEventListener('message', handleMessage, false); // global method used in html-inline elements

window.closeWindow = function () {
  setTimeout(function () {
    window.postMessage({
      type: POPUP.CLOSE_WINDOW
    }, window.location.origin);
    window.close();
  }, 100);
};