"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.selectDevice = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _builder = require("../../message/builder");

var UI = _interopRequireWildcard(require("../../constants/ui"));

var POPUP = _interopRequireWildcard(require("../../constants/popup"));

var _errors = require("../../constants/errors");

var _common = require("./common");

var _DataManager = _interopRequireDefault(require("../../data/DataManager"));

var _browserUtils = require("../../env/browser/browserUtils");

var _installBridge = require("./installBridge");

var _lang = _interopRequireDefault(require("../../data/lang"));

var _locale = _interopRequireDefault(require("../../utils/locale"));

var _copyToClipboard = _interopRequireDefault(require("copy-to-clipboard"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var initWebUsbButton = function initWebUsbButton(webusb, showLoader) {
  if (!webusb) return;

  var webusbContainer = _common.container.getElementsByClassName('webusb')[0];

  webusbContainer.style.display = 'flex';
  var button = webusbContainer.getElementsByTagName('button')[0];
  var usb = _common.iframe ? _common.iframe.clientInformation.usb : null;

  var onClick = /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (usb) {
                _context.next = 3;
                break;
              }

              window.postMessage({
                type: POPUP.EXTENSION_USB_PERMISSIONS
              }, window.location.origin);
              return _context.abrupt("return");

            case 3:
              _context.prev = 3;
              _context.next = 6;
              return usb.requestDevice({
                filters: _DataManager["default"].getConfig().webusb
              });

            case 6:
              if (showLoader) {
                (0, _common.showView)('loader');
              }

              _context.next = 11;
              break;

            case 9:
              _context.prev = 9;
              _context.t0 = _context["catch"](3);

            case 11:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[3, 9]]);
    }));

    return function onClick() {
      return _ref.apply(this, arguments);
    };
  }();

  button.onclick = onClick;
};

var selectDevice = function selectDevice(payload) {
  if (!payload) return;

  if (!payload.devices || !Array.isArray(payload.devices) || payload.devices.length === 0) {
    // No device connected
    // if (!iframe) {
    //     initInstallBridgeView()
    //     return
    // }
    (0, _common.showView)('connect');
    var connectEl = document.getElementsByClassName('connect')[0];
    var continueEl = connectEl.getElementsByClassName('continue')[0];
    continueEl.innerHTML = _lang["default"][_locale["default"]]['connectToContinue']; // initWebUsbButton(payload.webusb, true);

    return;
  }

  (0, _common.showView)('select-device'); // initWebUsbButton(payload.webusb, false);
  // If only 'remember device for now' toggle and no webusb button is available
  // show it right under the table

  if (!payload.webusb) {
    var wrapper = _common.container.getElementsByClassName('wrapper')[0];

    wrapper.style.justifyContent = 'normal';
  } // Populate device list


  var deviceList = _common.container.getElementsByClassName('select-device-list')[0]; // deviceList.innerHTML = '';


  var rememberCheckbox = _common.container.getElementsByClassName('remember-device')[0];

  var orderMap = {
    'unacquired': 1,
    'unreadable': 2
  }; // Show readable devices first

  payload.devices.sort(function (d1, d2) {
    var order1 = orderMap[d1.type] || 0;
    var order2 = orderMap[d2.type] || 0;
    return order1 - order2;
  });
  var normalDeviceTitle, normalDeviceDesc, descClassName;

  if (payload.devices.length) {
    var lastDeviceType = payload.devices[payload.devices.length - 1].type;

    if (lastDeviceType === 'unacquired' || lastDeviceType === 'unreadable') {
      normalDeviceTitle = _lang["default"][_locale["default"]]['connectedDevice'];
      normalDeviceDesc = _lang["default"][_locale["default"]]['paired'];
      descClassName = 'paired';
    } else {
      normalDeviceTitle = _lang["default"][_locale["default"]]['findDevice'];
      normalDeviceDesc = _lang["default"][_locale["default"]]['connectDevice'];
      descClassName = 'desc';
    }
  }

  var normalTitle, unacquiredTitle, unreadableTitle;
  var inactiveIndex = 0,
      unreadableIndex = 0;
  var url = _locale["default"] === 'zh' || _locale["default"] === 'zh_TW' ? 'https://help.tokenpocket.pro/cn/contact-us/contact-methods' : 'https://help.tokenpocket.pro/en/contact-us/contact-methods';
  var selectDeviceEl = document.getElementsByClassName('select-device')[0];
  var mask = selectDeviceEl.getElementsByClassName('command-popup--mask')[0];
  var titleEl = mask.getElementsByClassName('title')[0];
  var span1El = mask.getElementsByClassName('span1')[0];
  var span2El = mask.getElementsByClassName('span2')[0];
  var contactEl = mask.getElementsByClassName('contact')[0];
  var urlEl = mask.getElementsByClassName('url')[0];
  var copyEl = mask.getElementsByClassName('copy')[0];
  var btnEl = mask.getElementsByClassName('btn')[0];
  var copiedEl = selectDeviceEl.getElementsByClassName('copied')[0];
  titleEl.innerText = _lang["default"][_locale["default"]]['unreadableDevice'];
  span1El.innerText = _lang["default"][_locale["default"]]['unreadableDeviceDesc1'];
  contactEl.innerText = _lang["default"][_locale["default"]]['unreadableDeviceDesc2'];
  span2El.innerText = _lang["default"][_locale["default"]]['unreadableDeviceDesc3'];
  copyEl.innerText = _lang["default"][_locale["default"]]['copy'];
  btnEl.innerText = _lang["default"][_locale["default"]]['ok'];
  copiedEl.innerText = _lang["default"][_locale["default"]]['copied'];
  contactEl.setAttribute('href', url);

  copyEl.onclick = function () {
    (0, _copyToClipboard["default"])(urlEl.innerText);
    copiedEl.style.display = 'block';
    setTimeout(function () {
      copiedEl.style.display = 'none';
    }, 2000);
  };

  btnEl.onclick = function () {
    mask.style.display = 'none';
  };

  payload.devices.forEach(function (device) {
    if (device.type === 'unacquired' && !unacquiredTitle) {
      unacquiredTitle = _lang["default"][_locale["default"]]['inactiveDevice'];

      var _titleEl = document.createElement('p');

      _titleEl.classList.add('title');

      _titleEl.innerText = unacquiredTitle;
      deviceList.appendChild(_titleEl);
    } else if (device.type === 'unreadable' && !unreadableTitle) {
      unreadableTitle = _lang["default"][_locale["default"]]['unreadableDevice'];

      var _titleEl2 = document.createElement('p');

      _titleEl2.classList.add('title');

      _titleEl2.innerText = unreadableTitle;
      deviceList.appendChild(_titleEl2);
    } else if (!normalTitle) {
      normalTitle = normalDeviceTitle;

      var _titleEl3 = document.createElement('p');

      _titleEl3.classList.add('title');

      _titleEl3.innerText = normalTitle;
      deviceList.appendChild(_titleEl3);
    }

    var deviceButton = document.createElement('button');
    deviceButton.className = 'list';

    if (device.type !== 'unreadable') {
      deviceButton.addEventListener('click', function () {
        (0, _common.postMessage)((0, _builder.UiMessage)(UI.RECEIVE_DEVICE, {
          remember: rememberCheckbox && rememberCheckbox.checked,
          device: device
        }));
        (0, _common.showView)('loader');
      });
    }

    var deviceIcon = document.createElement('span');
    deviceIcon.className = 'icon';
    var deviceName = document.createElement('span');
    deviceName.className = 'device-name';
    deviceName.textContent = device.label;
    var wrapper = document.createElement('div');
    wrapper.className = 'wrapper';
    wrapper.appendChild(deviceIcon);
    wrapper.appendChild(deviceName);
    deviceButton.appendChild(wrapper);

    if (device.type !== 'unreadable') {
      var rightWrapper = document.createElement('div');
      rightWrapper.className = 'right-wrapper';
      var descEl = document.createElement('span');
      var enterEl = document.createElement('div');

      if (device.type === 'unacquired') {
        descEl.innerText = _lang["default"][_locale["default"]]['activeDevice'];
        descEl.className = 'desc';
      } else {
        descEl.innerText = normalDeviceDesc;
        descEl.className = descClassName;
      }

      enterEl.className = 'enter';
      rightWrapper.appendChild(descEl);
      rightWrapper.appendChild(enterEl);
      deviceButton.appendChild(rightWrapper);
    }

    var explanation;
    var mask = selectDeviceEl.getElementsByClassName('command-popup--mask')[0];

    if (device.type !== 'acquired' || device.status === 'occupied') {
      deviceButton.classList.add('device-explain'); // handle unreadable device

      if (device.type === 'unreadable') {
        explanation = document.createElement('p');
        explanation.className = 'explain';
        var os = (0, _browserUtils.getOS)(); // linux + LIBUSB_ERROR handling
        // if (device.error.indexOf(LIBUSB_ERROR_MESSAGE) >= 0) {          // 要改回去

        if (os === 'linux' && device.error.indexOf(_errors.LIBUSB_ERROR_MESSAGE) >= 0) {
          var span1 = document.createElement('span');
          var span2 = document.createElement('span');
          span1.innerText = _lang["default"][_locale["default"]]['linuxUnreadable'];
          span2.innerText = _lang["default"][_locale["default"]]['viewDetail'];
          span2.className = 'view';
          explanation.appendChild(span1);
          explanation.appendChild(span2);

          span2.onclick = function () {
            mask.style.display = 'block';
          };
        } else {
          explanation.innerText = _lang["default"][_locale["default"]]['unreadableDeviceWarn'];
        }

        deviceButton.disabled = true;
        deviceIcon.classList.add('unknown');
        unreadableIndex += 1;
        deviceName.textContent = "Unrecognized device" + unreadableIndex;
      }

      if (device.type === 'unacquired' || device.status === 'occupied') {
        inactiveIndex += 1;
        deviceName.textContent = "Inactive device" + inactiveIndex;
        deviceButton.classList.add('unacquired'); // explanation.classList.add('unacquired');

        deviceIcon.classList.add('unacquired');

        if (device.type === 'acquired') {
          inactiveIndex -= 1;
          deviceName.textContent = device.label;
        }
      }
    }

    deviceList.appendChild(deviceButton);

    if (explanation) {
      deviceList.appendChild(explanation);
    }
  });
  var helpEl = selectDeviceEl.getElementsByClassName('help')[0];
  var spanEl = helpEl.getElementsByTagName('span')[0];
  var aEl = helpEl.getElementsByTagName('a')[0];
  spanEl.innerText = _lang["default"][_locale["default"]]['connectProblem'];
  aEl.innerText = _lang["default"][_locale["default"]]['getHelp'];
  aEl.setAttribute('href', url);
};

exports.selectDevice = selectDevice;